#!/usr/bin/env python3
"""Draw random trees"""

import math
import random
import tkinter


MAXX = 512
MAXY = 512
BUD_SIZE = 3
MAXITER = 8

def random_triple(level):
    """Return 3 random numbrs for tree growth"""
    level += 1
    result = (25+random.uniform(0, MAXY/(5.0*level)),
              random.uniform(-math.pi/3.0, math.pi/3.0),
              random.random())
    return result

def grow_branch(branch, level):
    """Given a branch, grow two branches from it:
    its continuation and a side branch"""
    dy = branch[1][1] - branch[0][1]
    dx = branch[1][0] - branch[0][0]
    branch_angle = math.atan2(dy, dx)
    branch_params = random_triple(level)
    branch_cont = ((branch[1][0], branch[1][1]), \
    	(branch[1][0] + \
         branch_params[0] * math.cos(branch_angle),\
        (branch[1][1] + \
    	 branch_params[0] * math.sin(branch_angle))))
    branch_side_angle = branch_angle + branch_params[1]
    branch_side_start = (branch[0][0] + branch_params[2] * \
                         (branch[1][0] - branch[0][0]),
                         branch[0][1] + branch_params[2] * \
                         (branch[1][1] - branch[0][1]))
    branch_side = (branch_side_start, \
    	    (branch_side_start[0] + \
             branch_params[0] * math.cos(branch_side_angle),\
             branch_side_start[1] + \
             branch_params[0] * math.sin(branch_side_angle)))
    return [branch_cont, branch_side]

def draw_branch(branch):
    """Draws a branch: a line and two buds at its ends.
    The input of form ((x0,y0),(x1,y1))"""
    CANVAS.create_line(branch[0][0] + MAXX/2.0,
                       MAXY - branch[0][1],
                       branch[1][0] + MAXX/2.0,
                       MAXY - branch[1][1],
                       fill="darkblue", width=2.0)
    CANVAS.create_oval(-BUD_SIZE + branch[0][0] + MAXX/2.0,
                       -BUD_SIZE+ MAXY - branch[0][1],
                       BUD_SIZE + branch[0][0] + MAXX/2.0,
                       BUD_SIZE + MAXY - branch[0][1],
                       outline="darkblue")
    CANVAS.create_oval(-BUD_SIZE + branch[1][0] + MAXX/2.0,
                       -BUD_SIZE+ MAXY - branch[1][1],
                       BUD_SIZE + branch[1][0] + MAXX/2.0,
                       BUD_SIZE + MAXY - branch[1][1],
                       outline="darkblue")

#Now the main part. We open a canavas
MASTER = tkinter.Tk()
CANVAS = tkinter.Canvas(MASTER, width=MAXX, height=MAXY)
CANVAS.pack()
#Loop of drawing trees
while True:
    trunk = ((0., 20.), (10 * random.random(), 100.))
    branches = [trunk]
    for the_level in range(MAXITER):
        new_branches = []
        for the_branch in branches:
            draw_branch(the_branch)
            new_branches += grow_branch(the_branch, the_level)
        branches = new_branches
    print("Press ENTER for a next tree or enter 'q' to quit")
    cont = input()
    if cont in {'q', 'x'}:
        MASTER.destroy()
        exit(0)
    CANVAS.delete("all")
